@@ -14,50 +14,179 @@
     [clojure.string :as str]
     [clojure.tools.deps.alpha.util.concurrent :as concurrent]
     [clojure.tools.deps.alpha.util.dir :as dir]
+    [clojure.tools.deps.alpha.util.io :as io]
+    [clojure.tools.deps.alpha.util.session :as session]
     [clojure.tools.deps.alpha.extensions :as ext]
-
-    ;; Load extensions
-    [clojure.tools.deps.alpha.extensions.maven]
-    [clojure.tools.deps.alpha.extensions.local]
-    [clojure.tools.deps.alpha.extensions.git]
-    [clojure.tools.deps.alpha.extensions.deps]
-    [clojure.tools.deps.alpha.extensions.pom])
+    [clojure.walk :as walk])
   (:import
+    [clojure.lang EdnReader$ReaderException]
     [clojure.lang PersistentQueue]
-    [java.io File]))
+    [java.io File InputStreamReader BufferedReader]))
 
-(set! *warn-on-reflection* true)
+;(set! *warn-on-reflection* true)
+
+;;;; deps.edn reading
+
+(defn- io-err
+  ^Throwable [fmt ^File f]
+  (let [path (.getAbsolutePath f)]
+    (ex-info (format fmt path) {:path path})))
+
+(defn- slurp-edn-map
+  "Read the file specified by the path-segments, slurp it, and read it as edn."
+  [^File f]
+  (let [val (try (io/slurp-edn f)
+                 (catch EdnReader$ReaderException e (throw (io-err (str (.getMessage e) " (%s)") f)))
+                 (catch RuntimeException t
+                   (if (str/starts-with? (.getMessage t) "EOF while reading")
+                     (throw (io-err "Error reading edn, delimiter unmatched (%s)" f))
+                     (throw (io-err (str "Error reading edn. " (.getMessage t) " (%s)") f)))))]
+    (if (map? val)
+      val
+      (throw (io-err "Expected edn map in: %s" f)))))
+
+;; all this canonicalization is deprecated and will eventually be removed
+
+(defn- canonicalize-sym
+  ([s]
+   (canonicalize-sym s nil))
+  ([s file-name]
+   (if (simple-symbol? s)
+     (let [cs (as-> (name s) n (symbol n n))]
+       (io/printerrln "DEPRECATED: Libs must be qualified, change" s "=>" cs
+         (if file-name (str "(" file-name ")") ""))
+       cs)
+     s)))
+
+(defn- canonicalize-exclusions
+  [{:keys [exclusions] :as coord} file-name]
+  (if (seq (filter simple-symbol? exclusions))
+    (assoc coord :exclusions (mapv #(canonicalize-sym % file-name) exclusions))
+    coord))
+
+(defn- canonicalize-dep-map
+  [deps-map file-name]
+  (when deps-map
+    (reduce-kv (fn [acc lib coord]
+                 (let [new-lib (if (simple-symbol? lib) (canonicalize-sym lib file-name) lib)
+                       new-coord (canonicalize-exclusions coord file-name)]
+                   (assoc acc new-lib new-coord)))
+      {} deps-map)))
+
+(defn- canonicalize-all-syms
+  ([deps-edn]
+   (canonicalize-all-syms deps-edn nil))
+  ([deps-edn file-name]
+   (walk/postwalk
+     (fn [x]
+       (if (map? x)
+         (reduce (fn [xr k]
+                   (if-let [xm (get xr k)]
+                     (assoc xr k (canonicalize-dep-map xm file-name))
+                     xr))
+           x #{:deps :default-deps :override-deps :extra-deps :classpath-overrides})
+         x))
+     deps-edn)))
+
+(defn slurp-deps
+  "Read a single deps.edn file from disk and canonicalize symbols,
+  return a deps map."
+  [^File dep-file]
+  (-> dep-file slurp-edn-map (canonicalize-all-syms (.getPath dep-file))))
+
+(defn root-deps
+  "Read the root deps.edn resource from the classpath at the path
+  clojure/tools/deps/deps.edn"
+  []
+  (let [url (jio/resource "clojure/tools/deps/deps.edn")]
+    (io/read-edn (BufferedReader. (InputStreamReader. (.openStream url))))))
+
+(defn user-deps-path
+  "Use the same logic as clj to calculate the location of the user deps.edn.
+  Note that it's possible no file may exist at this location."
+  []
+  (let [config-env (System/getenv "CLJ_CONFIG")
+        xdg-env (System/getenv "XDG_CONFIG_HOME")
+        home (System/getProperty "user.home")
+        config-dir (cond config-env config-env
+                         xdg-env (str xdg-env File/separator "clojure")
+                         :else (str home File/separator ".clojure"))]
+    (str config-dir File/separator "deps.edn")))
+
+(defn find-edn-maps
+  "Finds and returns standard deps edn maps in a map with keys
+    :root-edn, :user-edn, :project-edn
+  If no project-edn is supplied, use the deps.edn in current directory"
+  ([]
+   (find-edn-maps nil))
+  ([project-edn-file]
+   (let [user-loc (jio/file (user-deps-path))
+         project-loc (jio/file (if project-edn-file project-edn-file (str dir/*the-dir* File/separator "deps.edn")))]
+     (cond-> {:root-edn (root-deps)}
+       (.exists user-loc) (assoc :user-edn (slurp-deps user-loc))
+       (.exists project-loc) (assoc :project-edn (slurp-deps project-loc))))))
+
+(defn- merge-or-replace
+  "If maps, merge, otherwise replace"
+  [& vals]
+  (when (some identity vals)
+    (reduce (fn [ret val]
+              (if (and (map? ret) (map? val))
+                (merge ret val)
+                (or val ret)))
+      nil vals)))
+
+(defn merge-edns
+  "Merge multiple deps edn maps from left to right into a single deps edn map."
+  [deps-edn-maps]
+  (apply merge-with merge-or-replace (remove nil? deps-edn-maps)))
+
+;;;; Aliases
+
+;; per-key binary merge-with rules
+
+(def ^:private last-wins (comp last #(remove nil? %) vector))
+(def ^:private append (comp vec concat))
+(def ^:private append-unique (comp vec distinct concat))
 
 (def ^:private merge-alias-rules
-  {:deps merge
+  {:deps merge ;; FUTURE: remove
+   :replace-deps merge ;; formerly :deps
    :extra-deps merge
    :override-deps merge
    :default-deps merge
    :classpath-overrides merge
-   :paths (comp vec distinct concat)
-   :extra-paths (comp vec distinct concat)
-   :jvm-opts (comp vec concat)
-   :main-opts (comp last #(remove nil? %) vector)})
+   :paths append-unique ;; FUTURE: remove
+   :replace-paths append-unique ;; formerly :paths
+   :extra-paths append-unique
+   :jvm-opts append
+   :main-opts last-wins
+   :exec-fn last-wins
+   :exec-args merge-or-replace
+   :ns-aliases merge
+   :ns-default last-wins})
 
-(defn- choose-rule [alias-key]
+(defn- choose-rule [alias-key val]
   (or (merge-alias-rules alias-key)
-    (throw (ex-info (format "Unknown alias key: %s" alias-key) {:key alias-key}))))
+    (if (map? val)
+      merge
+      (fn [_v1 v2] v2))))
 
 (defn- merge-alias-maps
   "Like merge-with, but using custom per-alias-key merge function"
   [& ms]
   (reduce
     #(reduce
-       (fn [m [k v]] (update m k (choose-rule k) v))
+       (fn [m [k v]] (update m k (choose-rule k v) v))
        %1 %2)
     {} ms))
 
 (defn combine-aliases
   "Find, read, and combine alias maps identified by alias keywords from
-  a deps configuration into a single args map."
-  [deps alias-kws]
+  a deps edn map into a single args map."
+  [edn-map alias-kws]
   (->> alias-kws
-    (map #(get-in deps [:aliases %]))
+    (map #(get-in edn-map [:aliases %]))
     (apply merge-alias-maps)))
 
 (defn lib-location
@@ -66,11 +195,7 @@
   [lib coord deps-config]
   (ext/lib-location lib coord deps-config))
 
-(defn- canonicalize-deps
-  [deps config]
-  (reduce
-    (fn [m [lib coord]] (conj m (ext/canonicalize lib coord config)))
-    [] deps))
+;;;; expand-deps
 
 ;; exclusions tree
 
@@ -146,14 +271,19 @@
   (get-in vmap [lib :versions (selected-version vmap lib)]))
 
 (defn- parent-missing?
-  "Is parent path now missing from the selected lib/versions?
-  This can happen if a newer version is found, orphaning enqueued children."
-  [vmap path]
-  (when (seq path)
-    (let [parent-lib (last path)
-          parent-path (vec (butlast path))
-          {:keys [paths select]} (get vmap parent-lib)]
-      (not (contains? (get paths select) parent-path)))))
+  "Is any part of the parent path missing from the selected lib/versions?
+  This can happen if a newer version was found, orphaning previously selected children."
+  [vmap parent-path]
+  (when (seq parent-path)
+    (loop [path parent-path]
+      (if (seq path)
+        (let [lib (last path)
+              check-path (vec (butlast path))
+              {:keys [paths select]} (get vmap lib)]
+          (if (contains? (get paths select) check-path)
+            (recur check-path)
+            true))
+        false))))
 
 (defn- dominates?
   "Is new-coord newer than old-coord?"
@@ -208,7 +338,11 @@
     :else
     {:include false, :reason :older-version, :vmap vmap}))
 
-;; expand-deps
+(defn- canonicalize-deps
+  [deps config]
+  (reduce
+    (fn [m [lib coord]] (conj m (ext/canonicalize lib coord config)))
+    [] deps))
 
 (defn- trace+
   "Add an entry to the trace if needed"
@@ -275,13 +409,29 @@
               (trace+ trace? trace parents lib coord use-coord coord-id override-coord include reason)))
           (cond-> version-map trace? (with-meta {:trace {:log trace, :vmap version-map, :exclusions exclusions}})))))))
 
-(defn- lib-paths
+(defn- cut-orphans
+  "Remove any selected lib that does not have a selected parent path"
   [version-map]
-  (reduce
-    (fn [ret [lib {:keys [select versions paths]}]]
+  (reduce-kv
+    (fn [vmap lib {:keys [select paths]}]
+      (let [parent-paths (get paths select)]
+        ;(println "check parent-missing" lib parent-paths)
+        (if (every? #(parent-missing? version-map %) parent-paths)
+          (dissoc vmap lib)
+          vmap)))
+    version-map version-map))
+
+(defn- lib-paths
+  "Convert version map to lib map"
+  [version-map]
+  (reduce-kv
+    (fn [ret lib {:keys [select versions paths]}]
       (let [coord (get versions select)
-            paths (->> (get paths select) (map last) (remove nil?) vec)]
-        (assoc ret lib (cond-> coord (seq paths) (assoc :dependents paths)))))
+            parent-paths (get paths select)
+            parents (->> parent-paths (map last) (remove nil?) vec)]
+        (assoc ret lib (cond-> coord
+                         (seq parents) (assoc :dependents parents)
+                         (seq parent-paths) (assoc :parents parent-paths)))))
     {} version-map))
 
 (defn- download-libs
@@ -312,27 +462,26 @@
     :extra-deps - a map from lib to coord of deps to add to the main deps
     :override-deps - a map from lib to coord of coord to use instead of those in the graph
     :default-deps - a map from lib to coord of deps to use if no coord specified
-
-  settings is an optional map of settings:
     :trace - boolean. If true, the returned lib map will have metadata with :trace log
-    :threads - int. If provided, sets the number of concurrent download threads
+    :threads - long. If provided, sets the number of concurrent download threads
 
   Returns a lib map (map of lib to coordinate chosen)."
+  {:arglists '([deps-map args-map])}
   ([deps-map args-map]
-    (resolve-deps deps-map args-map nil))
-  ([deps-map args-map settings]
-   (let [{:keys [extra-deps default-deps override-deps]} args-map
-         n (if-let [threads (:threads settings)]
-             (Long/parseLong threads)
-             concurrent/processors)
+   (let [{:keys [extra-deps default-deps override-deps threads trace]} args-map
+         n (or threads concurrent/processors)
          executor (concurrent/new-executor n)
          deps (merge (:deps deps-map) extra-deps)
          version-map (-> deps
                        (canonicalize-deps deps-map)
-                       (expand-deps default-deps override-deps deps-map executor (:trace settings)))
+                       (expand-deps default-deps override-deps deps-map executor trace)
+                       cut-orphans)
          lib-map (lib-paths version-map)
          lib-map' (download-libs executor lib-map deps-map)]
-     (with-meta lib-map' (meta version-map)))))
+     (with-meta lib-map' (meta version-map))))
+  ;; deprecated arity, retained for backwards compatibility
+  ([deps-map args-map settings]
+   (resolve-deps deps-map (merge args-map settings))))
 
 (defn- make-tree
   [lib-map]
@@ -359,22 +508,67 @@
       (doseq [[lib coord] tree :when (-> coord :dependents nil?)]
         (print-node lib "")))))
 
-(defn- make-classpath-roots
-  "Takes a lib map, and a set of explicit paths. Extracts the paths for each chosen
-  lib coordinate, and assembles a classpath string using the system path separator.
-  The classpath-args is a map with keys that can be used to modify the classpath
-  building operation:
+(defn- chase-key
+  "Given an aliases set and a keyword k, return a flattened vector of path
+  entries for that k, resolving recursively if needed, or nil."
+  [aliases k]
+  (let [path-coll (get aliases k)]
+    (when (seq path-coll)
+      (into [] (mapcat #(if (string? %) [[% {:path-key k}]] (chase-key aliases %))) path-coll))))
 
+(defn- flatten-paths
+  [{:keys [paths aliases] :as deps-edn-map} {:keys [extra-paths] :as classpath-args}]
+  (let [aliases' (assoc aliases :paths paths :extra-paths extra-paths)]
+    (into [] (mapcat #(chase-key aliases' %)) (remove nil? [:extra-paths :paths]))))
+
+(defn- tree-paths
+  "Given a lib map, return a vector of all vector paths to included libs in the tree.
+  Libs are often included at multiple paths."
+  [lib-map]
+  (loop [paths []
+         [lib & libs] (keys lib-map)]
+    (if lib
+      (let [parent-paths (:parents (get lib-map lib))]
+        (recur
+          (reduce (fn [paths parent-path] (conj paths (conj parent-path lib))) paths parent-paths)
+          libs))
+      paths)))
+
+(defn- sort-paths
+  [lib-paths]
+  "Given a vector of lib paths, sort in canonical order -
+  top of tree to bottom, alpha sort at same level"
+  (->> lib-paths sort (sort-by count) vec))
+
+(defn- flatten-libs
+  [lib-map {:keys [classpath-overrides] :as classpath-args}]
+  (let [override-libs (merge-with (fn [coord path] (assoc coord :paths [path])) lib-map classpath-overrides)
+        lib-order (->> override-libs tree-paths sort-paths (map peek) distinct)
+        lib-paths (mapcat
+                    #(map vector (get-in override-libs [% :paths]) (repeat {:lib-name %}))
+                    lib-order)]
+    lib-paths))
+
+(defn make-classpath-map
+  "Takes a merged deps edn map and a lib map. Extracts the paths for each chosen
+  lib coordinate, and assembles a classpath map. The classpath-args is a map with
+  keys that can be used to modify the classpath building operation:
     :extra-paths - extra classpath paths to add to the classpath
     :classpath-overrides - a map of lib to path, where path is used instead of the coord's paths
 
-  Returns the classpath as a vector of string paths."
-  [lib-map paths {:keys [classpath-overrides extra-paths] :as classpath-args}]
-  (let [libs (merge-with (fn [coord path] (assoc coord :paths [path])) lib-map classpath-overrides)
-        lib-paths (mapcat :paths (vals libs))]
-    (remove str/blank? (concat extra-paths paths lib-paths))))
+  Returns a map:
+    :classpath map of path entry (string) to a map describing where its from,  either a :lib-name or :path-key entry.
+    :classpath-roots coll of the classpath keys in classpath order"
+  [{:keys [paths aliases] :as deps-edn-map} lib-map classpath-args]
+  (let [flat-paths (flatten-paths deps-edn-map classpath-args)
+        flat-libs (flatten-libs lib-map classpath-args)
+        all-paths (concat flat-paths flat-libs)
+        cp (mapv first all-paths)
+        cp-map (reduce (fn [m [path-str src]] (assoc m path-str src)) {} all-paths)]
+    {:classpath-roots cp
+     :classpath cp-map}))
 
-(defn- join-classpath
+(defn join-classpath
   "Takes a coll of string classpath roots and creates a platform sensitive classpath"
   [roots]
   (str/join File/pathSeparator roots))
@@ -389,19 +583,82 @@
     :classpath-overrides - a map of lib to path, where path is used instead of the coord's paths
 
   Returns the classpath as a string."
+  {:deprecated "0.9.745"}
   [lib-map paths classpath-args]
-  (-> (make-classpath-roots lib-map paths classpath-args) join-classpath))
+  (-> (make-classpath-map {:paths paths} lib-map classpath-args) :classpath-roots join-classpath))
+
+(defn tool
+  "Transform project edn for tool by applying tool args (keys = :paths, :deps) and
+  returning an updated project edn."
+  [project-edn tool-args]
+  (let [{:keys [replace-deps replace-paths deps paths]} tool-args]
+    ;; FUTURE: stop supporting :deps + :paths in aliases
+    (when deps
+      (io/printerrln "WARNING: Use of :deps in aliases is deprecated - use :replace-deps instead"))
+    (when paths
+      (io/printerrln "WARNING: Use of :paths in aliases is deprecated - use :replace-paths instead"))
+    (cond-> project-edn
+      (or deps replace-deps) (merge {:deps (merge deps replace-deps)})
+      (or paths replace-paths) (merge {:paths (vec (concat paths replace-paths))}))))
+
+(defn calc-basis
+  "Calculates and returns the runtime basis from a master deps edn map, modifying
+   resolve-deps and make-classpath args as needed.
+
+    master-edn - a master deps edn map
+    args - an optional map of arguments to constituent steps, keys:
+      :resolve-args - map of args to resolve-deps, with possible keys:
+        :extra-deps
+        :override-deps
+        :default-deps
+        :threads - number of threads to use during deps resolution
+        :trace - flag to record a trace log
+      :classpath-args - map of args to make-classpath-map, with possible keys:
+        :extra-paths
+        :classpath-overrides
+
+  Returns the runtime basis, which is the initial deps edn map plus these keys:
+    :resolve-args - the resolve args passed in, if any
+    :classpath-args - the classpath args passed in, if any
+    :libs - lib map, per resolve-deps
+    :classpath - classpath map per make-classpath-map
+    :classpath-roots - vector of paths in classpath order"
+  ([master-edn]
+   (calc-basis master-edn nil))
+  ([master-edn {:keys [resolve-args classpath-args]}]
+   (session/with-session
+     (let [libs (resolve-deps master-edn resolve-args)
+           cp (make-classpath-map master-edn libs classpath-args)]
+       (cond->
+         (merge master-edn {:libs libs} cp)
+         resolve-args (assoc :resolve-args resolve-args)
+         classpath-args (assoc :classpath-args classpath-args))))))
+
+;; Load extensions
+(load "/clojure/tools/deps/alpha/extensions/maven")
+(load "/clojure/tools/deps/alpha/extensions/local")
+(load "/clojure/tools/deps/alpha/extensions/git")
+(load "/clojure/tools/deps/alpha/extensions/deps")
+(load "/clojure/tools/deps/alpha/extensions/pom")
 
 (comment
   (require '[clojure.tools.deps.alpha.util.maven :as mvn])
 
   (def ex-svc (concurrent/new-executor 2))
 
+  (calc-basis
+    {:mvn/repos mvn/standard-repos
+     :paths ["src" "test"]
+     :deps {'org.clojure/clojure {:mvn/version "1.9.0"}
+            'org.clojure/core.memoize {:mvn/version "0.5.8"}}
+     :aliases {:x {:extra-paths ["a" "b"]}}}
+    {:classpath-args {:extra-paths ["a" "b"]}})
+
   (expand-deps {'org.clojure/clojure {:mvn/version "1.9.0"}}
     nil nil {:mvn/repos mvn/standard-repos} ex-svc true)
 
   (expand-deps {'org.apache.xmlgraphics/batik-transcoder {:mvn/version "1.7"}}
-               nil nil {:mvn/repos mvn/standard-repos} ex-svc true)
+    nil nil {:mvn/repos mvn/standard-repos} ex-svc true)
 
   (expand-deps {'org.clojure/clojure {:mvn/version "1.9.0"}
                 'org.clojure/core.memoize {:mvn/version "0.5.8"}}
@@ -424,17 +681,19 @@
                           'org.clojure/core.memoize {:mvn/version "0.5.8"}}
                    :mvn/repos mvn/standard-repos} nil))
 
-  (make-classpath
+  (make-classpath-map
+    {:paths ["src"]}
     (resolve-deps {:deps {'org.clojure/clojure {:mvn/version "1.9.0-alpha17"}
                           'org.clojure/core.memoize {:mvn/version "0.5.8"}}
-                   :mvn/repos mvn/standard-repos} nil) ["src"] {:extra-paths ["test"]})
+                   :mvn/repos mvn/standard-repos} nil)
+    {:extra-paths ["test"]})
 
   (clojure.pprint/pprint
     (resolve-deps {:deps {'org.clojure/tools.analyzer.jvm {:mvn/version "0.6.9"}}
                    :mvn/repos mvn/standard-repos} nil))
 
   (clojure.pprint/pprint
-    (resolve-deps {:deps {'cheshire {:mvn/version "5.7.0"}}
+    (resolve-deps {:deps {'cheshire/cheshire {:mvn/version "5.7.0"}}
                    :mvn/repos mvn/standard-repos} nil))
 
   ;; top deps win
@@ -456,47 +715,50 @@
       {:extra-deps {'org.clojure/tools.gitlibs {:mvn/version "0.2.64"}}}))
 
   ;; override-deps
-  (make-classpath
+  (make-classpath-map
+    {:paths ["src"]}
     (resolve-deps
       {:deps {'org.clojure/core.memoize {:mvn/version "0.5.8"}}
        :mvn/repos mvn/standard-repos}
       {:override-deps {'org.clojure/clojure {:mvn/version "1.3.0"}}})
-    ["src"] nil)
+    nil)
 
-  (make-classpath
+  (make-classpath-map
+    {:paths ["src"]}
     (resolve-deps
       {:deps {'org.clojure/tools.deps.alpha {:mvn/version "0.1.40"}
               'org.clojure/clojure {:mvn/version "1.9.0-alpha17"}}
-       :mvn/repos mvn/standard-repos} nil) nil nil)
+       :mvn/repos mvn/standard-repos} nil) nil)
 
   ;; extra paths
-  (make-classpath
+  (make-classpath-map
+    {:paths ["src"]}
     (resolve-deps
       {:deps {'org.clojure/clojure {:mvn/version "1.10.0"}}
        :mvn/repos mvn/standard-repos} nil)
-    ["src"]
     {:extra-paths ["extra"]})
 
   ;; classpath overrides
-  (make-classpath
+  (make-classpath-map
+    {}
     (resolve-deps
       {:deps {'org.clojure/tools.deps.alpha {:mvn/version "0.1.40"}
               'org.clojure/clojure {:mvn/version "1.9.0-alpha17"}}
-       :mvn/repos mvn/standard-repos} nil) nil
+       :mvn/repos mvn/standard-repos} nil)
     '{:classpath-overrides {org.clojure/clojure "foo"}})
 
   (resolve-deps
     {:deps '{org.clojure/clojure {:mvn/version "1.9.0"}
              org.clojure/clojurescript {:mvn/version "1.9.946"}
-             reagent {:mvn/version "0.6.0"}}
+             reagent/reagent {:mvn/version "0.6.0"}}
      :mvn/repos mvn/standard-repos}
     nil)
 
   ;; err case
-  (resolve-deps {:deps {'bogus {:mvn/version "1.2.3"}}
+  (resolve-deps {:deps {'bogus/bogus {:mvn/version "1.2.3"}}
                  :mvn/repos mvn/standard-repos} nil)
 
-  (resolve-deps {:deps {'bogus "1.2.3"}
+  (resolve-deps {:deps {'bogus/bogus "1.2.3"}
                  :mvn/repos mvn/standard-repos} nil)
 
   (require
@@ -504,8 +766,8 @@
     '[clojure.tools.deps.alpha.extensions.deps])
 
   (resolve-deps
-    {:deps {'foo {:git/url "https://github.com/clojure/core.async.git"
-                  :sha "ecea2539a724a415b15e50f12815b4ab115cfd35"}}}
+    {:deps {'foo/bar {:git/url "https://github.com/clojure/core.async.git"
+                      :sha "ecea2539a724a415b15e50f12815b4ab115cfd35"}}}
     nil)
 
   (require '[clojure.tools.deps.alpha.util.session :as session])
@@ -519,4 +781,4 @@
           nil))
       nil))
 
-  )
\ No newline at end of file
+  )
